<?php
/**
 * Provides public helper methods.
 *
 * @package GoFetch/Frontend
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Helper class.
 */
class GoFetch_Frontend {

	public function __construct() {

		if ( is_admin() ) {
			return;
		}

		add_filter( 'wp', array( $this, 'single_goft_job' ) );
		add_filter( 'wp_head', array( $this, 'maybe_no_robots' ), 1 );
		add_action( 'goft_wpjm_single_goft_job', array( $this, 'single_job_page_hooks' ) );

		add_filter( 'goft_wpjm_read_more', array( $this, 'remove_more_on_scraped_descriptions' ), 10, 3 );
		add_filter( 'goft_wpjm_post_insert', array( $this, 'maybe_expire_job' ), 10, 2 );
	}

	/**
	 * Actions that should run on the single job page.
	 */
	public function single_job_page_hooks( $post ) {
		add_filter( 'get_canonical_url', array( $this, 'maybe_add_rel_canonical_to_header' ), 10, 2 );
		add_filter( 'wpseo_canonical', array( $this, 'maybe_add_rel_canonical_to_header' ), 10 );
		add_filter( 'the_content', array( __CLASS__, 'goft_the_job_description' ), 10 );

	}

	/**
	 * Executes several functions on the job description
	 */
	public static function goft_the_job_description( $content ) {
		$content = self::auto_format_plain_text( $content );
		$content = self::remove_empty_paragraphs( $content );
		$content = self::append_external_url( $content );
		$content = self::append_source( $content );
		return apply_filters( 'goft_the_job_description', $content );
	}

	/**
	 * Triggers an action when a users is viewing a single job page.
	 */
	public function single_goft_job() {
		global $post;

		if ( ! GoFetch_Jobs()->is_single_goft_job_page() ) {
			return;
		}
		do_action( 'goft_wpjm_single_goft_job', $post );
	}

	/**
	 * Adds a 'noindex' meta tag to disable jobs indexing for some providers
	 */
	public function maybe_no_robots() {
		global $post, $goft_wpjm_options;

		if ( ! GoFetch_Jobs()->is_single_goft_job_page() ) {
			return;
		}


		if ( $goft_wpjm_options->block_search_indexing ) {
			wp_no_robots();
			return;
		}

		$source_data = get_post_meta( $post->ID, '_goft_source_data', true );

		do_action( 'goft_no_robots', $source_data['feed_url'] );
	}

	/**
	 * 	Don't display the [...] (more) link if the job description was scraped.
	 */
	public function remove_more_on_scraped_descriptions( $more, $content, $link ) {
		global $post;

		$meta = get_post_meta( $post->ID, '_goft_wpjm_other', true );

		if ( ! empty( $meta['scrape']['description'] ) ) {
			return '';
		}

		return $more;
	}

	/**
	 * Expire job that explicitly have the 'expired' attribute set to 'true'.
	 */
	public function maybe_expire_job( $post_arr, $item ) {

		if ( ! isset( $item['expired'] ) || ! ( (bool) $item['expired'] ) ) {
			return $post_arr;
		}

		$post_array['post_status'] = 'expired';

		return $post_arr;
	}

	/**
	 * Optionally replaces the canonical URL for imported jobs with the job external link.
	 */
	public function maybe_add_rel_canonical_to_header( $canonical_url, $post = 0 ) {
		global $goft_wpjm_options, $post;

		if ( ! $post ) {
			global $post;
		}

		if ( $goft_wpjm_options->canonical_links ) {
			$canonical_url = get_post_meta( $post->ID, '_goft_external_link', true );
		}
		return $canonical_url;
	}

	/**
	 * Auto format plain text descriptions without any paragraphs.
	 */
	public static function auto_format_plain_text( $description ) {
		global $post, $goft_wpjm_options;

		$paragraphs = preg_split('#(\r\n?|\n)+#', $description);

		if ( ! $goft_wpjm_options->auto_format_descriptions || count( $paragraphs ) > apply_filters( 'goft_wpjm_auto_format_plain_text_paragraphs', $goft_wpjm_options->format_descriptions_paragraph_check, $post ) ) {
			return $description;
		}

		$sentences = preg_split( '/[.?!]/', $description, -1, PREG_SPLIT_DELIM_CAPTURE );

		$split = apply_filters( 'goft_wpjm_auto_format_plain_text_nth', $goft_wpjm_options->format_descriptions_stops_split, $post );

		$i = 0;

		$formatted_desc = '';

		foreach ( $sentences as $key => $sentence ) {

			// Avoid adding extra punctuation on last sentence.
			if ( $key >= count( $sentences ) - 1 ) {
				$formatted_desc .= "{$sentence}";
				continue;
			}

			$formatted_desc .= "{$sentence}";
			$formatted_desc  = rtrim( $formatted_desc, '.' ) . '. ';

			if ( ++$i % $split ==  0 ) {
				$formatted_desc .= '<br/><br/>';
				$i = 0;
			}
		}

		$ul       = apply_filters( 'goft_wpjm_auto_format_lists', '*' );
		$ul_match = apply_filters( 'goft_wpjm_auto_format_list_match', '\*' );

		if ( $ul ) {
			$formatted_desc = preg_replace( '/\s' . "{$ul_match}" . '\s/', "<br/> {$ul} ", $formatted_desc );
		}
		return $formatted_desc;
	}

	/**
	 * Remove empty paragraphs.
	 */
	public static function remove_empty_paragraphs( $content ) {
		global $post;

		if ( ! apply_filters( 'goft_wpjm_remove_empty_paragraphs', true, $post ) ) {
			return $content;
		}

		$content = preg_replace( '/<p[^>]*>(?:\s+|(?:&nbsp;)+|(?:&#013;)|(?:<br\s*\/?>)+)*<\/p>/', '', $content);

		return $content;
	}


	/**
	 * Append the job details external URL to the post content.
	 */
	public static function append_external_url( $content ) {
		global $post, $goft_wpjm_options;

		$append_external_url = $goft_wpjm_options->allow_visitors_apply || is_user_logged_in();

		if ( ! apply_filters( 'goft_wpjm_append_external_url', $append_external_url ) ) {
			return $content;
		}

		// Skip earlier if the content already contains the '[...]' with the external url.
		if ( false !== stripos( $content, '[&#8230;]' ) ) {
			return $content;
		}

		$source_data = get_post_meta( $post->ID, '_goft_source_data', true );

		$link = get_post_meta( $post->ID, '_goft_external_link', true );
		$link = GoFetch_Importer::add_query_args( $source_data, $link );

		// If the content is wrapped in <p> tags make sure the <a> is added inside it.
		$content_inline = '/p>' === trim( substr( $content, -4 ) ) ? substr( $content, 0, -5 ) : $content;

		$read_more = apply_filters( 'goft_wpjm_read_more', $goft_wpjm_options->read_more_text, $content, $link );

		if ( $read_more ) {

			$link_atts = apply_filters( 'goft_wpjm_read_more_link_attributes', array(
				'class'  => 'goftj-exernal-link',
				'href'   => $link,
				'rel'    => 'nofollow',
				'target' => '_blank',
			), $post );

			$content = sprintf( '%1$s %2$s', $content_inline, html( 'a', $link_atts, $read_more ) ) . '</p>';
		} else {
			$content = $content_inline;
		}
		return $content;
	}

	/**
	 * Append the source job URL to the post content.
	 */
	public static function append_source( $content ) {
		global $post;

		if ( ! apply_filters( 'goft_wpjm_append_source_data', true ) ) {
			return $content;
		}

		$source_data = get_post_meta( $post->ID, '_goft_source_data', true );

		if ( empty( $source_data['name'] ) || empty( $source_data['website'] ) ) {
			return $content;
		}

?>
		<style type="text/css">
			.entry-content a.goftj-logo-exernal-link {
				box-shadow: none;
			}
			.goftj-source-logo {
				height: 32px;
			}
			.goftj-logo-exernal-link {
				display: -webkit-inline-box;
				vertical-align: -webkit-baseline-middle;
			}
		</style>
<?php
		$external_link = get_post_meta( $post->ID, '_goft_external_link', true );

		$link = GoFetch_Importer::add_query_args( $source_data, $external_link );

		if ( ! empty( $source_data['logo'] ) ) {

			$atts = apply_filters( 'goft_wpjm_source_image_attributes', array(
				'src'    => esc_url( $source_data['logo'] ),
				'rel'    => 'nofollow',
				'title'  => esc_attr( $source_data['name'] ),
			) );
			$source = html( 'img class="goftj-source-logo"', $atts, '&nbsp;' );

		} else {
			$source = html( 'span class="goftj-source"', $source_data['name'] );
		}

		if ( $link ) {

			$atts = apply_filters( 'goft_wpjm_source_link_attributes', array(
				'class'  => 'goftj-logo-exernal-link',
				'rel'    => 'nofollow',
				'href'   => esc_url( $link ),
				'title'  => esc_attr( $source_data['name'] ),
				'target' => '_blank',
			), $post );

			$source = html( 'a', $atts, $source );
		}

		$source = html( 'p', sprintf( __( '<em class="goft-source">%1$s</em> %2$s', 'gofetch-wpjm' ), __( 'Source:', 'gofetch-wpjm' ), $source ) );

		return $content . $source;
	}

	/**
	 * The alternative application markup with extra link attributes.
	 */
	public static function application_details_url( $apply ) {
		global $post, $goft_wpjm_options;

		$link = $apply->url;

		$atts = apply_filters( 'goft_wpjm_source_link_attributes', array(
			'class'  => 'goftj-logo-exernal-link',
			'rel'    => 'nofollow',
			'href'   => $link,
			'target' => '_blank',
		), $post );

		if ( ! $goft_wpjm_options->apply_to_job_text ) {
			$text = __( 'To apply, please visit the following URL:', 'gofetch-wpjm' );
		}

		if ( $goft_wpjm_options->apply_to_job_hide_link ) {
			$text = html( 'p', html( 'a', array_map( 'esc_attr', $atts ), $goft_wpjm_options->apply_to_job_text ) );
		} else {
			$text = html( 'p', $goft_wpjm_options->apply_to_job_text . html( 'a', $atts, esc_url( $link ) . '&rarr;' ) );
		}
		echo apply_filters( 'goft_wpjm_source_link', $text, $atts, $apply->url );
	}

}

new GoFetch_Frontend();
