<?php
/**
 * Import functionality related with setup options.
 *
 * @package GoFetch/Dynamic/Admin/Dynamic Importer
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

require_once dirname( GOFT_WPJM_PLUGIN_FILE ) . '/includes/class-gofetch-importer.php';

/**
 * WPJM specific import functionality.
 */
class GoFetch_Dynamic_Import extends GoFetch_Importer {

	/**
	 * @var The single instance of the class.
	 */
	protected static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function __construct() {
		add_filter( 'goft_wpjm_item_meta_value', array( $this, 'replace_item_meta_placeholders' ), 10, 5 );
		add_filter( 'goft_wpjm_import_item_params', array( $this, 'default_item_meta' ), 10, 2 );
		add_action( 'goft_wpjm_after_insert_job', array( $this, 'maybe_remove_expiry_date' ) );
		add_filter( 'goft_wpjm_field_mapping_default', array( __CLASS__, 'default_field_mapping' ), 10, 3 );
		add_filter( 'goft_wpjm_feed_results', array( __CLASS__, 'hidden_fields' ) );
	}

	/**
	 * Setup default values for meta fields.
	 */
	public function default_item_meta( $params, $items ) {
		global $goft_wpjm_options;

		$defaults = array(
			$goft_wpjm_options->setup_field_application => '',
		);
		$params['meta'] = wp_parse_args( $params['meta'], $defaults );

		return $params;
	}

	/**
	 * Replaces string placeholders with valid data on a given meta key.
	 */
	public function replace_item_meta_placeholders( $meta_value, $meta_key, $item, $post_id, $params ) {
		global $goft_wpjm_options;

		switch ( $meta_key ) {

			case $goft_wpjm_options->setup_field_application:
				// Use the external job URL for application only if the application field is empty.
				if ( ! $meta_value ) {
					$meta_value = self::add_query_args( $params, $item['link'] );
				}
				break;

			// Fix any coordinates using comma as decimal point.
			case $goft_wpjm_options->setup_field_latitude:
			case $goft_wpjm_options->setup_field_longitude:
				$meta_value = str_replace( ',', '.', $meta_value );
				break;

			case $goft_wpjm_options->setup_field_expiration:
				$curr_date = date( 'Y-m-d', current_time( 'timestamp' ) );

				// Get the value provided by the user (if greater then current date) or default to settings duration.
				if ( $meta_value && strtotime( $meta_value ) > strtotime( $curr_date ) ) {
					return $meta_value;
				}
				$meta_value = self::get_expire_date();
				break;

		}
		return $meta_value;
	}

	/**
	 * Remove the expiry date meta on non-published jobs and let WPJM calculate it when the user publishes the job.
	 */
	public function maybe_remove_expiry_date( $post_id ) {
		global $goft_wpjm_options;

		if ( 'publish' !== get_post_status( $post_id ) ) {
			delete_post_meta( $post_id, $goft_wpjm_options->setup_field_expiration );
		}
	}

	/**
	 * Calculates the jobs expire date considering the WPJM job duration option.
	 * Leave empty if not set
	 */
	public static function get_expire_date( $date = '' ) {
		global $goft_wpjm_options;

		if ( $duration = $goft_wpjm_options->jobs_duration ) {
			$date = $date ? $date: current_time( 'mysql' );
			return date( 'Y-m-d', strtotime( $date . ' +' . absint( $duration ) . ' days' ) );
		}
	}


	/**
	 * Rerieve a list of default field mappings.
	 */
	public static function scrape_fields() {
		return apply_filters( 'goft_wpjm_scrape_fields', array(
			'logo',
			'logo_html',
			'description',
			'company',
			'location',
		) );
	}

	/**
	 * Rerieve a list of default field mappings.
	 */
	public static function default_mappings() {
		$fields = apply_filters( 'goft_wpjm_feed_fields', array(
			'description',
			'author',
			'creator',
			'city',
			'location',
			'country',
			'state',
			'latitude',
			'lat',
			'longitude',
			'long',
			'lng',
			'company',
			'company_name',
			'website',
			'site',
			'company_url',
			'company_site',
			'job_type',
			'type',
			'job_category',
			'category',
		) );
		foreach ( $fields as $field ) {
			$mappings[ $field ] = self::default_field_mapping( $field, $field );
		}
		return $mappings;
	}

	/**
	 * the default field mappings for this provider.
	 */
	public static function default_field_mapping( $mapping, $field ) {
		global $goft_wpjm_options;

		switch ( $field ) {
			case 'description':
				$mapping = 'post_content';
				break;

			case 'author':
			case 'creator':
				$mapping = 'post_author';
				break;

			case 'city':
				$mapping = $goft_wpjm_options->setup_field_city;
				break;

			case 'location':
				$mapping = $goft_wpjm_options->setup_field_location;
				break;

			case 'country':
				$mapping = $goft_wpjm_options->setup_field_country;
				break;

			case 'state':
				$mapping = $goft_wpjm_options->setup_field_state;
				break;

			case 'latitude':
			case 'lat':
				$mapping = $goft_wpjm_options->setup_field_latitude;
				break;

			case 'longitude':
			case 'long':
			case 'lng':
				$mapping = $goft_wpjm_options->setup_field_longitude;
				break;

			case 'company':
			case 'company_name':
				$mapping = $goft_wpjm_options->setup_field_company_name;
				break;

			case 'website':
			case 'company_url':
			case 'site':
				$mapping = $goft_wpjm_options->setup_field_company_url;
				break;

			case 'logo':
			case 'company_logo':
				$mapping = $goft_wpjm_options->setup_field_company_logo;
				break;


			// Taxonomies.

			case 'job_type':
			case 'type':
				$mapping = $goft_wpjm_options->setup_job_type;
				break;

			case 'job_category':
			case 'category':
				$mapping = $goft_wpjm_options->setup_job_category;
				break;

		}
		return $mapping;
	}

	/**
	 * Fields hidden on the sample table.
	 */
	public static function hidden_fields( $results ) {

		$fields = array_keys( $results['sample_item'] );

		$hidden_fields = array(
			'pubDate',
			'pubdate',
			'guid',
		);

		foreach ( array_intersect( $fields, $hidden_fields ) as $field ) {
			unset( $results['sample_item'][ $field ] );
		}
		return $results;
	}

}

GoFetch_Dynamic_Import::instance();
